package de.ozgcloud.admin.setting;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.admin.setting.postfach.PostfachSettingBody;
import de.ozgcloud.admin.setting.postfach.PostfachSettingBodyTestFactory;

class SettingServiceTest {

	@InjectMocks
	@Spy
	private SettingService service;

	@Mock
	private SettingRepository repository;

	@Mock
	private SettingMapper mapper;

	private final PostfachSettingBody postfach = PostfachSettingBodyTestFactory.create();

	private static final String POSTFACH = "Postfach";
	private final Setting settingWithPostfach = SettingTestFactory.createBuilder().name(POSTFACH).settingBody(postfach).build();

	@Nested
	class TestGetAlfaSettingDTO {
		@Captor
		private ArgumentCaptor<AlfaSettingDTO.AlfaSettingDTOBuilder> alfaBuilderCaptor;

		private final AlfaSettingDTO alfaSetting = AlfaSettingDTOTestFactory.create();

		@BeforeEach
		void mockGetSettingWithPostfachFromDb() {
			doReturn(postfach).when(service).getSettingWithPostfachFromDb();
		}

		@Test
		void shouldCallGetSettingWithPostfachFromDb() {
			when(mapper.mapAlfaPostfach(any(), any())).thenReturn(AlfaSettingDTOTestFactory.createBuilder());

			service.getAlfaSettingDTO();

			verify(service).getSettingWithPostfachFromDb();
		}

		@Test
		void shouldCallMapperWithEmptyBuilderAndPostfach() {
			when(mapper.mapAlfaPostfach(any(), any())).thenReturn(AlfaSettingDTOTestFactory.createBuilder());

			service.getAlfaSettingDTO();

			verify(mapper).mapAlfaPostfach(alfaBuilderCaptor.capture(), eq(postfach));
			assertThat(alfaBuilderCaptor.getValue()).usingRecursiveComparison().isEqualTo(AlfaSettingDTO.builder());
		}

		@Test
		void shouldReturnBuiltAlfaSettingDTO() {
			when(mapper.mapAlfaPostfach(any(), any())).thenReturn(AlfaSettingDTOTestFactory.createBuilder());

			var resultSettingDTO = service.getAlfaSettingDTO();

			assertThat(resultSettingDTO).usingRecursiveComparison().isEqualTo(alfaSetting);
		}
	}

	@Nested
	class TestGetVorgangManagerSettingDTO {
		@Captor
		private ArgumentCaptor<VorgangManagerSettingDTO.VorgangManagerSettingDTOBuilder> vorgangManagerBuilderCaptor;

		private final VorgangManagerSettingDTO vorgangManagerSetting = VorgangManagerSettingDTOTestFactory.create();

		@BeforeEach
		void mockGetSettingWithPostfachFromDb() {
			doReturn(postfach).when(service).getSettingWithPostfachFromDb();
		}

		@BeforeEach
		void mockMapper() {
			when(mapper.mapVorgangManagerPostfach(any(), any())).thenReturn(VorgangManagerSettingDTOTestFactory.createBuilder());
		}

		@Test
		void shouldCallGetSettingWithPostfachFromDb() {
			service.getVorgangManagerSettingDTO();

			verify(service).getSettingWithPostfachFromDb();
		}

		@Test
		void shouldCallMapperWithEmptyBuilderAndPostfach() {
			service.getVorgangManagerSettingDTO();

			verify(mapper).mapVorgangManagerPostfach(vorgangManagerBuilderCaptor.capture(), eq(postfach));
			assertThat(vorgangManagerBuilderCaptor.getValue()).usingRecursiveComparison().isEqualTo(VorgangManagerSettingDTO.builder());
		}

		@Test
		void shouldReturnBuiltVorgangManagerSettingDTO() {
			var resultSettingDTO = service.getVorgangManagerSettingDTO();

			assertThat(resultSettingDTO).usingRecursiveComparison().isEqualTo(vorgangManagerSetting);
		}

	}

	@Nested
	class TestGetSettingWithPostfachFromDb {
		@Test
		void shouldCallRepository() {
			when(repository.findOneByName(POSTFACH)).thenReturn(Optional.of(settingWithPostfach));

			service.getSettingWithPostfachFromDb();

			verify(repository).findOneByName(POSTFACH);
		}

		@Test
		void shouldReturnPostfachSettingBody() {
			when(repository.findOneByName(POSTFACH)).thenReturn(Optional.of(settingWithPostfach));

			var returnedBody = service.getSettingWithPostfachFromDb();

			assertThat(returnedBody).isEqualTo(postfach);
		}

		@Test
		void shouldReturnEmptyPostfachSettingBodyForEmptySetting() {
			when(repository.findOneByName(POSTFACH)).thenReturn(Optional.empty());

			var returnedBody = service.getSettingWithPostfachFromDb();

			assertThat(returnedBody).usingRecursiveComparison().isEqualTo(PostfachSettingBody.builder().build());
		}
	}

}
